from django.conf import settings
from achievements.models import Achievement
from achievements.utils import construct_callback, check_achievement_plain
from achievements.signals import achievement_registered
import logging

logger = logging.getLogger(__name__)

class AchievementEngine(object):
    def register_achievement(self, cls):
        """ Register a new achievement into the engine and database"""
        (obj, created) = Achievement.objects.get_or_create(key=cls.key,  defaults={
            'name': cls.name,
            'description': cls.description,
            'bonus': cls.bonus,
            'callback': construct_callback(cls)})
        if not created:
            # update the object if key didn't change :
            obj.name = cls.name
            obj.description = cls.description
            obj.bonus = cls.bonus
            obj.callback = construct_callback(cls)
            obj.save()
        # optional category
        if cls.category:
            obj.category = cls.category
            obj.save()
        achievement_registered.send(sender=self, achievement_class=cls)

    def check_achievement(self, user, key, *args, **kwargs):
        """
        Check synchronously or asynchronously if according to a specific context
        an achievement has been unlocked
        """
        if user and user.is_authenticated():
            if settings.ACHIEVEMENT_USE_CELERY:
                # do not try to import if celery is not defined
                from achievements.tasks import check_achievement_task
                check_achievement_task.delay(self, user, key, *args, **kwargs)
            else:
                check_achievement_plain(self, user, key, *args, **kwargs)
        else:
            logger.info("trying to check an achievement for an un-logged user")


# create the engine
engine = AchievementEngine()





from django.conf import settings

from .models import Achievement
from .utils import check_achievement_plain


class AutoAchievementChecker(object):
  def process_response(self, request, response):
    if not getattr(request, 'user'):
      return response

    user = request.user
    try:
      methods = [i.lower() for i in settings.ACHIEVEMENT_MIDDLEWARE_REQUEST_METHODS]
    except:
      methods = []

    if request.method.lower() in methods and user and user.is_authenticated():
      for obj in Achievement.objects.all().order_by('bonus'):
        check_achievement_plain(self, user, obj.key)

    return response




from distutils.core import setup

import sys

INSTALL_REQUIRES = ["django-appconf>=0.4.1"]

if sys.version_info < (2, 7):
  INSTALL_REQUIRES += ["importlib>=1.0.2"]


setup(
    name="django-achievements",
    packages=["achievements", "achievements.migrations", "achievements.templatetags"],
    version="0.1.0",
    description="Django application to deal with achievements and rewards unlocked by users",
    author="Olivier Girardot",
    author_email="ssaboum@gmail.com",
    url="https://github.com/ssaboum/django-achievements",
    install_requires=INSTALL_REQUIRES,
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
